# -*- coding: utf-8 -*-
"""
——————————————————————————————————————————————————————————————————————————————————————————
Clustering by Fast Search and Find of Density Peaks

输入数据：
    pd.DataFrame 每一行表示一个点

——————————————————————————————————————————————————————————————————————————————————————————
"""


# step 1. 计算距离矩阵

def cal_distance(xi, xj):
    pass


# step 2. 计算每一个点的局部密度
def cal_loc_density(ddi, dc):
    pass


# step 3. 计算每一个点与其他具有更高局部密度的点集的最小距离
def cal_min_distance(ddi, ld):
    pass


# step 4. 做决策图，确定类簇的数量
def decision_cluster_num():
    pass

# step 5. 归类
